package eu.ztsh.garmin.data

import com.mapbox.navigation.tripdata.maneuver.model.Maneuver
import com.mapbox.navigation.core.trip.session.LocationMatcherResult

class MapboxMapper {

    companion object {
        
        fun asDirection(maneuver: Maneuver): Direction {
            val direction = Direction()
            maneuver.primary.apply {
                when (this.type) {
                    "roundabout" -> {
                        direction.out = OutType.RightRoundabout
                    }
                    "fork" -> direction.out = OutType.LongerLane
                    "arrive" -> {
//                        flag = true
                    }
                    "turn" -> {
                        when (this.type) {
                            "straight" -> direction.angle = OutAngle.Straight
                        }
                    }
                }
                when (this.modifier) {
                    "right" -> {
                        when (this.type) {
                            "turn" -> direction.angle = OutAngle.Right
                            "roundabout" -> {
                                when (this.degrees) {
                                    137.0 -> direction.angle = OutAngle.EasyRight
                                    180.0 -> direction.angle = OutAngle.Straight
                                }
                            }
                            "off ramp" -> {
                                direction.angle = OutAngle.EasyRight
                                direction.out = OutType.LongerLane
                            }
                        }
                    }

                    "left" -> {
                        when (this.type) {
                            "turn" -> direction.angle = OutAngle.Left
                            "off ramp" -> {
                                direction.angle = OutAngle.EasyLeft
                                direction.out = OutType.LongerLane
                            }
                        }
                    }
                }
            }
            return direction
        }

        fun asDistance(maneuver: Maneuver): Distance {
            return maneuver.stepDistance.let { step ->
                step.distanceFormatter.formatDistance(step.distanceRemaining!!).split(" ").let {
                    Distance(
                        it[0].replace(',', '.').toDouble(),
                        when (it[1]) {
                            "m" -> Unit.Metres
                            "km" -> Unit.Kilometres
                            else -> Unit.Any
                        }
                    )
                }
            }
        }

        fun asLanes(maneuver: Maneuver): Lanes {
            val laneIterator = Lane.iterator()
            val outlines = mutableSetOf<Lane>()
            val lanes = mutableSetOf<Lane>()
            maneuver.laneGuidance?.apply {
                this.allLanes.reversed().let {
                    it.forEach{ indicator ->
                        val lane = if (laneIterator.hasNext()) laneIterator.next() else Lane.DotsLeft
                        if (lane == Lane.DotsLeft) {
                            outlines.add(Lane.DotsLeft)
                        } else {
                            outlines.add(lane)
                            if (indicator.isActive) {
                                lanes.add(lane)
                            }
                        }
                    }
                }
            }
            return Lanes(Arrows(lanes), Arrows(outlines))
        }

        fun map(locationMatcherResult: LocationMatcherResult): GarminLocation {
            val state = GarminLocation()
            // TODO: speed, limit, location?, bearing
            return state
        }

    }

}
